# -*- coding: UTF-8 -*-
from setuphelpers import *
from setupdevhelpers import *
import requests
import waptlicences

dist_dir = "distribution"
dist_path = makepath(programfiles, "Mozilla Firefox", dist_dir)
ext_dir = "extensions"
ext_path = makepath(dist_path, ext_dir)
policies_file = "policies.json"
policies_path = makepath(dist_path, policies_file)
list_pre_installed_lang = ["fr", "en-GB", "es-ES", "de", "it"]  # Reminder: 'en-US' will be native

def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    app_name = control.name

    arch_dict = {"x64": "win64", "x86": "win", "all": "win"}
    policies_content = json_load_file(policies_file)

    # Getting the latest version from official sources
    download_url = requests.head(f"https://download.mozilla.org/?product=firefox-latest-ssl&os={arch_dict[control.architecture]}&lang=en-US", proxies=proxies).headers["Location"]
    latest_bin = download_url.rsplit("/", 1)[-1].replace("%20", "_")
    latest_bin_extension = latest_bin.rsplit('.', 1)[-1]
    version = download_url.split("/")[-4]

    # Downloading the latest binaries
    print("Latest %s version is: %s" % (app_name, version))
    print("Download URL is: %s" % download_url)
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(download_url, latest_bin, proxies=proxies)
    else:
        print("Binary is present: %s" % latest_bin)

    expected_issuer = "Mozilla Corporation"
    sign_name = waptlicences.check_exe_signing_certificate(latest_bin)[0]
    if sign_name != expected_issuer:
        error(f'Bad issuer {sign_name} != {expected_issuer} ')

    # Changing the version of the package
    if Version(version) > Version(control.get_software_version()):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Deleting outdated binaries
    for f in glob.glob(f'*.{latest_bin_extension}'):
        if f != latest_bin:
            remove_file(f)

    # Remove existing extensions folder
    if isdir(ext_dir):
        remove_tree(ext_dir)

    download_firefox_language_pack(policies_content, proxies)
    download_firefox_dictionnary(policies_content, proxies)

    # Downloading ublock origin extensions
    wget("https://addons.mozilla.org/firefox/downloads/latest/ublock-origin/latest.xpi", makepath(ext_dir, "ublock-origin.xpi"), proxies=proxies)

    print(f"Creating {policies_file} file for language packs, dictionaries, extensions, and configuration")
    if isfile(policies_file):
        remove_file(policies_file)

    json_write_file(policies_file, policies_content, indent=4)

    # Validate or not the update-package-sources
    return package_updated

def download_firefox_language_pack(policies_content, proxies):

    print("\n### DOWNLOADING LANGUAGE PACK ###\n")

    url_api_lp = "https://services.addons.mozilla.org/api/v3/addons/language-tools/?app=firefox&type=language"
    print("API used for language packs is: " + url_api_lp)
    version_major = control.version.split("-")[0].split(".")[0]

    # Retrieved the download url of the language pack
    json_read = wgets(url_api_lp, proxies=proxies, as_json=True)
    
    for lang in list_pre_installed_lang:

        # Retrieve the download URL for the language pack
        url_lp = next((lp["url"] + "versions/" for lp in json_read["results"] if lp["target_locale"] == lang), None)

        if not url_lp:
            print(f"No URL found for language pack: {lang}")
            continue

        # Find the version-specific download link
        url_dl_lp = None
        for download_lp in bs_find_all(url_lp, "a", "class", "InstallButtonWrapper-download-link", proxies=proxies):
            if version_major in download_lp.get("href", ""):
                url_dl_lp = download_lp["href"]
                break

        # Check if a valid URL for the language pack was found
        if not url_dl_lp:
            print(f"No valid download link found for language pack: {lang}")
            continue

        lp_name = f"{lang}_language_pack.xpi"
        print(f"Download URL for language pack in {lang} is: {url_dl_lp}")
        wget(url_dl_lp, makepath(ext_dir, lp_name), proxies=proxies)

        policies_content["policies"]["ExtensionSettings"].update({
            f"langpack-{lang}@firefox.mozilla.org": {
                "install_url": f"file://{ext_path}\\{lang}_language_pack.xpi", 
                "installation_mode": "force_installed"
            }
        })

def download_firefox_dictionnary(policies_content, proxies):

    print("\n### DOWNLOADING DICTIONNARY ###\n")

    url_api_dict_lang = "https://services.addons.mozilla.org/api/v3/addons/language-tools/?app=firefox&type=dictionary"
    print("API used for language dictionaries is: " + url_api_dict_lang)

    json_read = wgets(url_api_dict_lang, proxies=proxies, as_json=True)

    for lang in list_pre_installed_lang:

        if lang == "de":
            lang = "de-DE"

        dict_lang = next((d for d in json_read["results"] if d["target_locale"] == lang), None)

        if not dict_lang:
            print(f"No dictionary found for language: {lang}")
            continue

        url_dl_dict_lang = f"https://addons.mozilla.org/firefox/downloads/latest/{dict_lang['slug']}/latest.xpi"
        ext_add_on_id = dict_lang["guid"]

        dict_lang_name = f"{lang}_dictionary.xpi"
        print(f"Download URL for dictionary in {lang} is: {url_dl_dict_lang}")
        wget(url_dl_dict_lang, makepath(ext_dir, dict_lang_name), proxies=proxies)

        policies_content["policies"]["ExtensionSettings"].update({
            ext_add_on_id: {
                "install_url": f"file://{ext_path}\\{dict_lang_name}", 
                "installation_mode": "force_installed"
                }
        })